function [L,R,U,S,V] = diag_iter(L,R,num_sing, num_iter)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% input:
% L : Squared symmetric matrix containing the left neighborhood information, can be computed as L = M*M'.
% R : Squared symmetric matrix containing the right neighborhood information, can be computed as R = M'*M;
%     It is assumed that one of L and R is low rank while the other is full rank. The main diagonal of the full rank
%     matrix is (partially) unknown and set to 0. The main diagonal of the low rank matrix is known.
% num_sing : Maximum number of singular values to use in SVD.
%
% output:
% L : The same squared symmetric matrix as in the input, but with integer (non-zero) diagonal entries
% R : The same squared symmetric matrix as in the input, but with integer (non-zero) diagonal entries
% U : Left singular matrix of M
% S : Diagonal matrix containing the (unsigned) singular values of M
% V : Right singular matrix of M
%
% description:
% This algorithm uses an iterative heuristic to update the main diagonal of L or R. (The one that is unknown).
% while at the same time obtaining the SVD decomposition of M (but with unsigned singular values).
%
% Written by Dora Erdos, Boston University, 2013
%
%

[n,~] = size(L);
[m,~] = size(R);

if num_sing == 0
	num_sing = min(n,m)
end

%If L is low rank and R is full rank
if n>m
	[U,SU] = eigs(L,num_sing);
	% diagonal matrix that in every iteration contains the current best guess for the main diagonal of R
	DR = zeros(m);
	Mhat = zeros(n,m);
	Mhat_new = zeros(n,m);
	% X masks the values that are known in diag(R)
	X = double(R>0);
	X = eye(m) - X.* eye(m);
	for i=1:num_iter
	% Halt after the algorithm converges
	% while norm(abs(Mhat-Mhatnew),'fro')<0.001
    		[V,SV] = eigs(R+DR,num_sing);
    		S = sqrt((SU+SV)/2);
    		Mhatnew = U*S*V';
		% DR contains estimates for the unknown diagonal values of R 
   		DR = X .* (Mhatnew'*Mhatnew);
    		Mhat = Mhatnew;
	end
	R = R + DR;
else
	[V,SV] = eigs(R,num_sing);
	DL = zeros(n);
	Mhat = zeros(n,m);
	Mhat_new = zeros(n,m);
	% X masks the values that are known in diag(R)
	X = double(L>0);
	X = eye(n) - X.* eye(n);
	for i=1:num_iter
	% Halt after the algorithm converges
	% while norm(abs(Mhat-Mhatnew),'fro')<0.001
    		[U,SU] = eigs(L+DL,num_sing);
    		S = sqrt((SU+SV)/2);
    		Mhatnew = U*S*V'; 
   		DL = X.*(Mhatnew*Mhatnew');
    		Mhat = Mhatnew;
	end
	L = L + DL;
end

