function[Mhat,U,SI,V,L,R] = hybrid(L,R,eff_rank,num_sing,thres,output_dir)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% input:
% L : Squared symmetric matrix containing the left neighborhood information, can be computed as L = M*M'.
%     If degrees of nodes are (partially) unknown, then the main diagonal of L is set to 0.
% R : Squared symmetric matrix containing the left neighborhood information.
%     If degrees of nodes are (partially) unknown, then the main diagonal of R is set to 0.
% eff_rank : Effective rank of the original matrix. If unknown, then set to 0.
% num_sing : Maximum number of singular values to use in SVD.
% thres : Rounding threshold to obtain binary matrix. To get the same algorithm as in our paper use thres = 0.5
% output_dir : full path to folder where output is written
%
% output:
% Mhat : Binary matrix estimating the true bi-adjacency matrix of the graph
% U : Estimated left singular matrix of M
% SI : Estimated diagonal matrix containing the singular values of M
% V : Estimated right singular matrix of M
% L : Same as input matrix but with the estimated main diagonal.
% R : Same as input matrix but with the estimated main diagonal.
%
% description:
% This function runs the hybrid algorithm described in [1].
% All resulting matrices are written to file. 
%
% Written by Dora Erdos, Boston University, 2013
%
% 
% If using this code please cite:
% [1]: Dora Erdos, Rainer Gemulla, Evimaria Terzi "Reconstructing Graphs from Neighborhood Data", 
% IEEE International Conference on Data Mining, 2012, Brussels, Belgium, December 2012.
%

[n,~] = size(L);
[m,~] = size(R);

%If the effective rank is unknown then set it to min(n,m)
if eff_rank == 0 | eff_rank > min(n,m)
  	eff_rank = min(n,m);
end



% If  the main diagonal of L and R is unknown
% run the LS algorithm on the lower rank of L and R
% run a modified version of iterative on the larger rank of L and R
% compute the estimated SVD of M
%number of iterations in iterative algorithm
num_iter = 100;
%if L is low rank and R full rank
if n>m
	L = diag_ls(L,eff_rank);
	[L,R,U,S,V] = diag_iter(L,R,num_sing,num_iter);
else
	R = diag_ls(R,eff_rank);	
	[L,R,U,S,V] = diag_iter(L,R,num_sing,num_iter);
end	
%write the resulting L and R matrices (now with non-zero main diagonals) to file
dlmwrite(strcat(output_dir,'L'),full(L),'\t');
dlmwrite( strcat(output_dir,'R'),full(R),'\t');

%write resulting SVD matrices to file
dlmwrite(strcat(output_dir,'U'),full(U),'\t');
dlmwrite(strcat(output_dir,'S'),full(S),'\t');
dlmwrite(strcat(output_dir,'V'),full(V),'\t');

% SI is the diagonal matrix containing the signed singular values in S
% where signs are computed based on our greedy heuristic
SI = greedy_signs(U,S,V,num_sing,thres);
dlmwrite(strcat(output_dir,'SI'),full(SI),'\t');

% Compute the binary aprroximation of the original matrix M matrix 
Mhat = mtx_round(U*SI*V',thres);
dlmwrite(strcat(output_dir,'M'),full(Mhat),'\t');


